{% extends "!layout.html" %}
{% block extrahead %}
    <link rel="icon" type="image/png" sizes="16x16" href="/_static/icon-16.png">
    <link rel="icon" type="image/png" sizes="32x32" href="/_static/icon-32.png">
    <link rel="icon" type="image/png" sizes="48x48" href="/_static/icon-48.png">
    <link rel="icon" type="image/png" sizes="64x64" href="/_static/icon-64.png">
    <link rel="icon" type="image/png" sizes="128x128" href="/_static/icon-128.png">
    <link rel="icon" type="image/png" sizes="256x256" href="/_static/icon-256.png">
    {{ super() }}
{% endblock %}